import sys
import os
import math

import getopt
import time
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

MAX_FRAME_RATE=60



import screenInfo
import mouseInfo
import level1
import stateAim
import stateEdit
import stateDialog
import fsmMgr
import options
import playerMgr
import tutorialProgress

import statePyWeekSplash
import stateBDGSplash
import stateMainMenu
import options
import soundMgr

def resize((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluOrtho2D(0,800,0,600)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def init():
    #glEnable(GL_TEXTURE_2D)
    #glShadeModel(GL_SMOOTH)
    glClearColor(0.0, 0.0, 0.0, 0.0)

    if options.gSmoothLines:
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    
        glLineWidth(2.5)
        if options.gAntiAlias is None:
            pass
        else:
            glEnable(GL_LINE_SMOOTH)
            glHint(GL_LINE_SMOOTH_HINT,options.gAntiAlias)


def update(ms):
    return fsmMgr.update(ms)

def draw():
    glClear(GL_COLOR_BUFFER_BIT)
    fsmMgr.draw()

def handleKey(k):
    if fsmMgr.handleKey(k):
        return

    #TODO: remove debug features, maybe
    elif k==K_s:
        level1.thisLevel.nextLevel()

    elif k==K_e:
        fsmMgr.pushState(stateEdit.StateEdit(None))

def handleMouseButton(bDown, button):
    fsmMgr.handleMouseButton(bDown, button)
    

def wrappedMain(flagDict):
    
    print "running with options:"
    for flagKey in flagDict.keys():
        print flagKey,":",flagDict[flagKey]
    print
    
    
    video_flags = OPENGL|DOUBLEBUF
    if not flagDict["windowed"]:
        video_flags = video_flags|FULLSCREEN

    print "about to init"
    pygame.init()

    sz=flagDict["width"],flagDict["height"]
    surface = pygame.display.set_mode(sz, video_flags)

    print "about to resize"
    resize(sz)


    print "about to set screenInfo"
    screenInfo.gWindowX,screenInfo.gWindowY=sz
    screenInfo.gScreenX,screenInfo.gScreenY=(800,600)

    pygame.display.set_caption("Woody Tigerbaum's Interplanetary Package Delivery - BDG_07_04","TWIPD")
    surface = pygame.image.load("Data/icon.png")
    pygame.display.set_icon(surface)

    pygame.mouse.set_visible(False)
    pygame.mouse.set_pos((sz[0]/2,sz[1]/2))

    options.gSmoothLines=flagDict["smoothLines"]
    options.gAntiAlias=flagDict["antiAlias"]

    options.gFullScreen=not flagDict["windowed"]
    options.gSound=flagDict["sound"]
    options.gInverseY=flagDict["invY"]

    init()

    soundMgr.loadSounds()
    
    fsmMgr.pushState(stateMainMenu.StateMainMenu(None))
    if not flagDict["skipIntro"]:
        fsmMgr.pushState(stateBDGSplash.StateBDGSplash(None))


    frames = 0
    ticks = pygame.time.get_ticks()
    myClock=pygame.time.Clock()

    myClock.tick(MAX_FRAME_RATE)

    timeToQuit=False
    isActive=True

    #the amount of milliseconds that a loop MUST run
    loopMinTime=1000/MAX_FRAME_RATE
    
    while 1:
        # note - this doesn't work well by itself. At the bottom of
        # the loop, I add in an additional sleep
        ms=myClock.tick(MAX_FRAME_RATE)
        loopTopTime=pygame.time.get_ticks()

        for event in pygame.event.get():
            if (event.type == QUIT):
                #(event.type == KEYDOWN and event.key == K_ESCAPE) or
                #(event.type == KEYDOWN and event.key == K_F4)):
                timeToQuit=True
                break

            if event.type == KEYDOWN:
                handleKey(event.key)

            elif event.type == MOUSEBUTTONDOWN:
                handleMouseButton(True, event.button)
            elif event.type == MOUSEBUTTONUP:
                handleMouseButton(False, event.button)

            elif event.type == KEYUP:
                global controlKey
                controlKey = ''
            elif ((sys.platform=="win32") and
                  (event.type==ACTIVEEVENT) and
                  (not options.gUnattended)):
                if event.state==6:
                    fsmMgr.handleActivate(True)
                    if flagDict["mouseGrab"]:
                        pygame.event.set_grab(True)
                    pygame.mouse.set_visible(False)
                    isActive=True
                elif event.state==2:
                    fsmMgr.handleActivate(False)
                    if flagDict["mouseGrab"]:
                        pygame.event.set_grab(False)
                    pygame.mouse.set_visible(True)
                    isActive=False

            
        if timeToQuit:
            break

        #if isActive:
        mouseInfo.readMouse()

        if isActive:
            #if not update(ms):
            if not update(loopMinTime):
                timeToQuit=True
        draw()
        pygame.display.flip()
        frames = frames+1
        loopBottomTime=pygame.time.get_ticks()
        loopElapsedTime=loopBottomTime-loopTopTime
        timeLeft=loopMinTime-loopElapsedTime
        if timeLeft>10:
            pygame.time.wait(timeLeft-5)
        else:
            pygame.time.wait(1)


    print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))

    if options.gSound:
        time.sleep(1.0)


def usage():
    print """
Possible Arguments or flags:

-w    windowed (default)
-f    fullscreen

-x <n> width of window (default: 800)
-y <n> height of window (default: 600)

-q    quiet: no sounds or music

-m    mouse stays in the window (default)
-f    mouse can leave the window

-h    print help (this message)
"""

def readOptFile(flagDict):
    f=open("options.txt")
    for optLine in f.readlines():
        optName,optVal=optLine.split(":")
        optVal=optVal.strip()
        print optName, optVal

        if optName=="fullscreen":
            flagDict['windowed']=(optVal=='0')
        elif optName=="resolution":
            rxs,rys=optVal.split(",")
            flagDict['width']=int(rxs)
            flagDict['height']=int(rys)
        if optName=="sound":
            flagDict['sound']=(optVal=='1')
        if optName=="invMouseY":
            flagDict['invY']=(optVal=='1')
        

            


def main(flagDict):

    try:
        readOptFile(flagDict)
    except IOError:
        pass
        
    while True:
        try:
            wrappedMain(flagDict)
            return
        except options.RestartSignal, signal:
            flagDict['windowed']=not signal.flags['FS']
            rx,ry=signal.flags['Res']
            flagDict['width']=rx
            flagDict['height']=ry
            flagDict['sound']=signal.flags['Snd']
            flagDict['invY']=signal.flags['InvY']
            flagDict['skipIntro']=True
    

if __name__ == '__main__':
    try:
        optlist, args = getopt.getopt(sys.argv[1:], 'wfx:y:qmfhl:zabc')
    except getopt.GetoptError, message:
        print message
        usage()
        sys.exit(1)

    flags={'windowed': True,
           'width': 800,
           'height': 600,
           'sound': True,
           'mouseGrab': False,
           'level': 1,
           'numHumans': 1,
           'numAI':0,
           'tutorial':True,
           'smoothLines':True,
           'antiAlias':GL_NICEST,
           'invY':False,
           'skipIntro':False,
          }

    for opt, arg in optlist:
        if opt == '-w': flags['windowed'] = True
        elif opt == '-f': flags['windowed'] = False
        elif opt == '-x': flags['width'] = int(arg)
        elif opt == '-y': flags['height'] = int(arg)
        elif opt == '-q': flags['sound'] = False
        elif opt == '-m': flags['mouseGrab'] = True
        elif opt == '-f': flags['mouseGrab'] = False
        elif opt == '-h': usage(); sys.exit(1)
        elif opt == '-l': flags['level'] = int(arg)
        elif opt == '-z': flags['smoothLines']=False
        elif opt == '-a': flags['antiAlias']=GL_NICEST
        elif opt == '-b': flags['antiAlias']=GL_FASTEST
        elif opt == '-c': flags['antiAlias']=None
        
    main(flags)

        
