from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *


import fsmMgr

import textDraw
import options

import soundMgr

DIALOG_WIDTH=500
DIALOG_HEIGHT=350

TITLE_HEIGHT = 16
TITLE_FONT_SIZE = 5
TITLE_PAD = 6

DIALOG_FONT_SIZE = 4
DIALOG_LINE_SPACING = 15

CLICK_MSG="Click To Continue."
CLICK_MSG_SIZE=4
CLICK_MSG_WIDTH=textDraw.stringWidth(CLICK_MSG_SIZE,CLICK_MSG)

class StateDialog(fsmMgr.State):
    def __init__(self, stringData):
        fsmMgr.State.__init__(self)

        self.clicked=False
        self.dialogTime=0.0

        self.title=stringData[0]

        self.titleWidth=textDraw.stringWidth(TITLE_FONT_SIZE,self.title)
        
        stringList=stringData[1].split("\n")

        self.stringList=[]
        for s in stringList:
            self.stringList.append((textDraw.stringWidth(DIALOG_FONT_SIZE,
                                                         s),
                                    s))

    def init(self):
        pass

    def update(self,dt):
        self.dialogTime += dt
        if options.gUnattended:
            fsmMgr.popState()

    def handleMouseButton(self, bDown, buttonIndex):
        if bDown==True and buttonIndex==1:
            self.clicked=True

        if bDown==False and buttonIndex==1 and self.clicked:
            fsmMgr.popState()
            soundMgr.playSound("ping")
        return True

    def handleKey(self, key):
        fsmMgr.popState()
        soundMgr.playSound("ping")
        return True

    def draw(self):
        self.parent.draw()

        cx=400
        cy=300

        hX=DIALOG_WIDTH/2
        hY=DIALOG_HEIGHT/2

        glBegin(GL_QUADS)
        glColor3f(0,0,0)
        glVertex(cx-hX,cy-hY)
        glVertex(cx+hX,cy-hY)
        glVertex(cx+hX,cy+hY)
        glVertex(cx-hX,cy+hY)
        glEnd()
        
        glColor3f(0.25,1,0.25)
        glBegin(GL_LINE_LOOP)
        glVertex(cx-hX,cy-hY)
        glVertex(cx+hX,cy-hY)
        glVertex(cx+hX,cy+hY)
        glVertex(cx-hX,cy+hY)
        
        glEnd()

        glBegin(GL_LINES)
        glVertex(cx-hX,
                 cy+hY-TITLE_HEIGHT-TITLE_PAD)

        glVertex(cx+hX,
                 cy+hY-TITLE_HEIGHT-TITLE_PAD)
        glEnd()
        
    
        textDraw.drawString(cx-self.titleWidth/2,
                            cy+hY-TITLE_HEIGHT,
                            (0.25,1,0.25),
                            TITLE_FONT_SIZE,
                            self.title)

        bodyTextHeight=DIALOG_LINE_SPACING*len(self.stringList)

        for i,lineInfo in enumerate(self.stringList):
            w,s=lineInfo
            textDraw.drawString(cx-w/2,
                                cy+bodyTextHeight/2-i*DIALOG_LINE_SPACING,
                                (0.25,1,0.25),
                                DIALOG_FONT_SIZE,
                                s)

        textDraw.drawString(cx-CLICK_MSG_WIDTH/2,
                            cy-hY+CLICK_MSG_SIZE*2,
                            (0.25,1,0.25),
                            CLICK_MSG_SIZE,
                            CLICK_MSG)
