import math
import sys

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

from numpy import array

import fsmMgr
import mouseInfo

import statePlayAd
import stateDialog
import level1
import stateAim
import edgesMenu
import widgets
import tutorialProgress
import playerMgr
import stateNewGameMenu       
import stateOptions
import soundMgr

#seconds to scan the height of the screen
SCAN_SPEED=1.6

SCAN_INTERVAL=4

BRIGHT_COLOR=(0.7, 1, 0.7)
DIM_COLOR=(0.0, 0.55, 0.0)
FADE_TIME=0.4

SHOW_DELAY=2.0


def playTutorial():
    tutorialProgress.startTutorial()
    playerMgr.initPlayers(1,0)
    level1.thisLevel.setLevel(1)
    
    fsmMgr.pushState(level1.thisLevel)
    fsmMgr.pushState(stateAim.StateAim(None))


    if tutorialProgress.gTutorial:
        fsmMgr.pushState(stateDialog.StateDialog(("Memo from Mr. Tigerbaum","""
You will be responsible for sending
packages to the planets marked with
an X.

Use the blue cursor to aim the
PACKAGE SLINGSHOT and click the left
mouse button.
""")))
    
        fsmMgr.pushState(stateDialog.StateDialog(("Memo from Mr. Tigerbaum","""
Attention New Employee:



Welcome to
Woody Tigerbaum's interplanetary
package delivery.

It is now time to begin your training.
You will be instructed in using our
patented PACKAGE SLINGSHOT.
""")))




    
    pass

def playGame():
    fsmMgr.popAll()
    fsmMgr.pushState(stateNewGameMenu.StateNewGameMenu(None))


def watchAd():
    fsmMgr.pushState(statePlayAd.StatePlayAd(None))

def optionsMenu():
    fsmMgr.popAll()
    fsmMgr.pushState(stateOptions.StateOptions(None))
    

class StateMainMenu(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)
        self.clicked=False

        self.mRasterPos=600
        self.splashTime=0.0
        self.rasterSpeed=600.0/SCAN_SPEED

        gEdgeList=edgesMenu.lines

        glEnableClientState(GL_VERTEX_ARRAY)
        glEnableClientState(GL_COLOR_ARRAY)

        self.lineColors={}

        self.verts=[]

        self.lineIndices=[]
        
        lines=gEdgeList.keys()
        lines.sort()
        for y in lines:
            self.lineColors[y]=(0,0,0)
            for x in gEdgeList[y]:
                self.verts.append((float(x),
                                   float(y),
                                   0.0))
                self.lineIndices.append(y)
        
        self.vertArray=array(self.verts,'f')
        self.vertArrayString=self.vertArray.tostring()
        self.colorArray=array((DIM_COLOR)*len(self.verts), 'f')
        self.colorArrayString=self.colorArray.tostring()

        glVertexPointer(3, GL_FLOAT, 0, self.vertArrayString)
        glColorPointer(3, GL_FLOAT, 0, self.colorArrayString)

        self.numVerts=len(self.verts)



        self.widgets=[widgets.Button(40,220,8,"Play Tutorial",playTutorial),
                      widgets.Button(40,170,8,"Play",playGame),
                      widgets.Button(40,120,8,"Intro Movie",watchAd),
                      widgets.Button(40,70,8,"Options",optionsMenu),
                      widgets.Button(40,20,8,"Quit",lambda:fsmMgr.popAll())
                      ]

        maxWidth=max(map(lambda x:x.width,self.widgets))
        for w in self.widgets:
            w.setWidth(maxWidth)



    def update(self, dt):

        dt=dt/1000.0
        self.splashTime+=dt

        self.mRasterPos-=self.rasterSpeed*dt

        mouseInfo.readMouse()
        for w in self.widgets:
            w.update(dt)

        if self.splashTime>(SCAN_SPEED+FADE_TIME):
            return
        
        for y in self.lineColors.keys():
            distFromRaster=y-self.mRasterPos
            secondsSinceRaster=distFromRaster/self.rasterSpeed

            if secondsSinceRaster>FADE_TIME:
                self.lineColors[y]=DIM_COLOR
            elif secondsSinceRaster<0:
                self.lineColors[y]=(0.0, 0.0, 0.0)
            else:
                br,bg,bb=BRIGHT_COLOR
                dr,dg,db=DIM_COLOR

                fracSinceRaster=secondsSinceRaster/FADE_TIME

                interpR=(dr-br)*fracSinceRaster+br
                interpG=(dg-bg)*fracSinceRaster+bg
                interpB=(db-bb)*fracSinceRaster+bb

                self.lineColors[y]=(interpR, interpG, interpB)
            

        for i,y in enumerate(self.lineIndices):
            cr,cg,cb=self.lineColors[y]
            self.colorArray[3*i]=cr
            self.colorArray[3*i+1]=cg
            self.colorArray[3*i+2]=cb
        self.colorArrayString=self.colorArray.tostring()
        glColorPointer(3, GL_FLOAT, 0, self.colorArrayString)
        
            

    def draw(self):
        glDrawArrays(GL_LINES, 0, self.numVerts)
        
        for w in self.widgets:
            w.draw()

    def handleMouseButton(self, bDown, buttonIndex):
        if bDown and buttonIndex==1:
            for w in self.widgets:
                if w.isMouseInMe():
                    soundMgr.playSound("ping")
                    w.callbackFunc()
        return True

    def done(self):
        fsmMgr.popState()

    

