import math
import sys

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import fsmMgr
import mouseInfo

import statePlayAd
import stateDialog
import level1
import stateAim
import edgesMenu
import widgets
import tutorialProgress
import playerMgr
import stateMainMenu
import screenInfo
import fade       
import stateMainMenu
import soundMgr

import tutorialProgress
import textDraw
import options

gWidgetList=None

newResDict={1:(640,480),
            2:(800,600),
            3:(1024,768),
            4:(1280,960)}



gOpts={"FS":False,
       "Res":2,
       "Snd":False,
       "InvY":False
       }

def initOpts():
    gOpts["FS"]=options.gFullScreen
    
    resToIndexDict={640:1,
                    800:2,
                    1024:3,
                    1280:4}
    gOpts["Res"]=resToIndexDict.get(screenInfo.gWindowX,2)

    gOpts["Snd"]=options.gSound

    gOpts["InvY"]=options.gInverseY

def updateOpts():
    gWidgetList[0].state =     gOpts["FS"]
    gWidgetList[1].state = not gOpts["FS"]

    gWidgetList[2].state = gOpts["Res"] == 1
    gWidgetList[3].state = gOpts["Res"] == 2
    gWidgetList[4].state = gOpts["Res"] == 3
    gWidgetList[5].state = gOpts["Res"] == 4

    gWidgetList[6].state =     gOpts["Snd"]
    gWidgetList[7].state = not gOpts["Snd"]

    gWidgetList[8].state =     gOpts["InvY"]
    gWidgetList[9].state = not gOpts["InvY"]

    


def pressFullscreen():
    gOpts["FS"]=True
    updateOpts()

def pressWindowed():
    gOpts["FS"]=False
    updateOpts()
                      
def pressRes1():
    gOpts["Res"]=1
    updateOpts()

def pressRes2():
    gOpts["Res"]=2
    updateOpts()

def pressRes3():
    gOpts["Res"]=3
    updateOpts()

def pressRes4():
    gOpts["Res"]=4
    updateOpts()

def pressSoundOn():
    gOpts["Snd"]=True
    updateOpts()

def pressSoundOff():
    gOpts["Snd"]=False
    updateOpts()

def pressInvOn():
    gOpts["InvY"]=True
    updateOpts()

def pressInvOff():
    gOpts["InvY"]=False
    updateOpts()

def pressCancel():
    fsmMgr.popAll()
    fsmMgr.pushState(stateMainMenu.StateMainMenu(None))

def pressSave():
    r=setState()
    saveState()
    if r:
        raiseRestart()
        
    fsmMgr.popAll()
    fsmMgr.pushState(stateMainMenu.StateMainMenu(None))

def pressOK():
    r=setState()
    if r:
        raiseRestart()
    fsmMgr.popAll()
    fsmMgr.pushState(stateMainMenu.StateMainMenu(None))

def setState():
    newRes=newResDict[gOpts["Res"]]

    bMustRestart=False

    wx,wy=screenInfo.gWindowX,screenInfo.gWindowY
    if newRes != (wx,wy):
        bMustRestart=True

    if gOpts["FS"]!=options.gFullScreen:
        bMustRestart=True
    
    
    options.gSound=gOpts["Snd"]
    options.gFullScreen=gOpts["FS"]
    options.gInverseY=gOpts["InvY"]

    return bMustRestart

def raiseRestart():
    newRes=newResDict[gOpts["Res"]]

    raise options.RestartSignal({"FS":options.gFullScreen,
                                 "Snd":options.gSound,
                                 "InvY":options.gInverseY,
                                 "Res":newRes})

    

def saveState():
    f=open("options.txt","wt")
    f.write("fullscreen:")
    if gOpts["FS"]:
        f.write("1")
    else:
        f.write("0")
    f.write("\n")

    f.write("resolution:")
    rx,ry=newResDict[gOpts["Res"]]
    f.write("%d,%d\n"%(rx,ry))

    f.write("sound:")
    if gOpts["Snd"]:
        f.write("1")
    else:
        f.write("0")
    f.write("\n")

    f.write("invMouseY:")
    if gOpts["InvY"]:
        f.write("1")
    else:
        f.write("0")
    f.write("\n")

    

    
                      
                      

def mid(a,b):
    return (a+b)/2



class StateOptions(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)

        self.clicked=False

        self.left=50
        self.right=750
        self.top=450
        self.bottom=150

        B=(50-32)/2
        global gWidgetList

        c2=400
        c1=mid(c2,self.left)
        c3=mid(c2,self.right)
        wHalf=c2-self.left-20
        wThird=(self.right-self.left)/3-20

        c4=mid(self.left,c1)
        c5=mid(c1,c2)
        c6=mid(c2,c3)
        c7=mid(c3,self.right)

        c8=self.left+(self.right-self.left)/6
        c9=self.left+5*(self.right-self.left)/6

        
        gWidgetList =[widgets.Button(c1,400+B,8,"FULLSCREEN",pressFullscreen),
                      widgets.Button(c3,400+B,8,"WINDOWED",pressWindowed),
                      
                      widgets.Button(c1,350+B,8,"640*480",pressRes1),
                      widgets.Button(c3,350+B,8,"800*600",pressRes2),
                      widgets.Button(c1,300+B,8,"1024*768",pressRes3),
                      widgets.Button(c3,300+B,8,"1280*960",pressRes4),

                      widgets.Button(c1,250+B,8,"SOUND ON",pressSoundOn),
                      widgets.Button(c3,250+B,8,"SOUND OFF",pressSoundOff),
                      
                      widgets.Button(c1,200+B,8,"INV MS Y",pressInvOn),
                      widgets.Button(c3,200+B,8,"NO INV MS Y",pressInvOff),

                      widgets.Button(c8,150+B,8,"CANCEL", pressCancel),
                      widgets.Button(c2,150+B,8,"SAVE", pressSave),
                      widgets.Button(c9,150+B,8,"OK", pressOK),
                      ]

        widths={0:wHalf,
                1:wHalf,
                2:wHalf,
                3:wHalf,
                4:wHalf,
                5:wHalf,
                6:wHalf,
                7:wHalf,
                8:wHalf,
                9:wHalf,
                10:wThird,
                11:wThird,
                12:wThird}

        for i,w in enumerate(gWidgetList):
            w.setWidth(widths[i])

        for w in gWidgetList:
            w.x=w.x-w.width/2

        for w in gWidgetList:
            w.makeDL()

        initOpts()
        updateOpts()

    def update(self, dt):
        dt=dt/1000.0

        mouseInfo.readMouse()
        for w in gWidgetList:
            w.update(dt)

    def draw(self):
        glColor3f(0,0,0)
        glBegin(GL_QUADS)
        glVertex(self.left,self.bottom)
        glVertex(self.right,self.bottom)
        glVertex(self.right,self.top)
        glVertex(self.left,self.top)
        glEnd()

        glColor3f(*fade.DIM_COLOR)
        glBegin(GL_LINE_LOOP)
        glVertex(self.left,self.bottom)
        glVertex(self.right,self.bottom)
        glVertex(self.right,self.top)
        glVertex(self.left,self.top)
        glEnd()
        
        for w in gWidgetList:
            w.draw()

        

            

    def handleMouseButton(self, bDown, buttonIndex):
        mouseInfo.readMouse()
        if bDown and buttonIndex==1:
            for w in gWidgetList:
                if w.isMouseInMe():
                    soundMgr.playSound("ping")
                    w.callbackFunc()
        return True

    def done(self):
        fsmMgr.popState()

    

