from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import fsmMgr
import textDraw

import mouseInfo
import fade

import soundMgr


class Button:
    def __init__(self, x, y, textSize, label, func):
        self.x=x
        self.y=y
        self.textSize=textSize
        self.label=label
        self.callbackFunc=func

        self.width=textDraw.stringWidth(textSize,label)+2*textSize
        self.fadeTime=0
        self.color=fade.DIM_COLOR

        self.state=False
        self.dispList=None
        self.makeDL()

    def __del__(self):
        if not (self.dispList is None):
            if glDeleteLists:
                glDeleteLists(self.dispList, 1)
            self.dispList=None

    def setWidth(self, w):
        self.width=w
        self.makeDL()

    def makeDL(self):
        if not (self.dispList is None):
            glDeleteLists(self.dispList,1)
        self.dispList=glGenLists(1)
        glNewList(self.dispList, GL_COMPILE)

        glBegin(GL_LINE_LOOP)
        glVertex(self.x,self.y)
        glVertex(self.x+self.width,self.y)
        glVertex(self.x+self.width,self.y+self.textSize*4)
        glVertex(self.x,self.y+self.textSize*4)
        glEnd()

        textDraw.drawCenteredString(self.x+self.width/2,
                                    self.y,
                                    self.color,
                                    self.textSize,
                                    self.label)

        glEndList()
        

    def draw(self):
        glColor(*self.color)

        glCallList(self.dispList)

        cursor_radius=20
        glColor3f(0, 0.7, 1)
        glBegin(GL_LINES)
        glVertex2f(mouseInfo.gMouseX-cursor_radius,
                   mouseInfo.gMouseY)
        glVertex2f(mouseInfo.gMouseX+cursor_radius,
                   mouseInfo.gMouseY)
        glVertex2f(mouseInfo.gMouseX,
                   mouseInfo.gMouseY-cursor_radius)
        glVertex2f(mouseInfo.gMouseX,
                   mouseInfo.gMouseY+cursor_radius)
        glEnd()

        
        
    def update(self, dt):
        if self.state:
            self.color=fade.BRIGHT_COLOR
        
        elif self.isMouseInMe():
            self.fadeTime+=dt
            self.color=fade.computeColor(0.0, self.fadeTime)
            
            
        else:
            self.fadeTime=0
            self.color=fade.DIM_COLOR

    def isMouseInMe(self):
        mx=mouseInfo.gMouseX
        my=mouseInfo.gMouseY

        return (mx>self.x and mx<self.x+self.width and
                my>self.y and my<self.y+4*self.textSize)
            
            
                 

    
    
